/**
 * package provides the user with methods to retrieve Duty Cycle
 * information or the current acceleration read from the duty
 * controlled outputs of an accelerometer.
 *
 * <P>objects can extend the accelerometer class to suit it for the
 * particular accelerometer they're using.
 *
 * <P><U><B> ACCELEROMETERS SUPPORTED BY THIS CLASS:</U></B>
 * <BR> Using one of the constructors which doesn't specify a scale factor
 * natively supports the popular accelerometers ADXL202EB and the Memsic2125.
 * Each of which have a scale factor of 12.5%.
 *
 * <P>However, this class can support any accelerometer that has duty cycle
 * modulated (DCM) outputs and uses the equation:
 * A(g) = (dutyFactor - .5) / scaleFactor
 * in order to calculate the gForce from the DCM outputs.  If the accelerometer
 * you are using has a scaleFactor other then 12.5% then you will need to
 * specify the scaleFactor when calling the constructor.
 *
 * Since floating point numbers aren't valid in the Javelin you need to take the
 * scale factor and multiply it by 1000.  For example if my accelerometer had a
 * scale factor of 4% then i have .04 * 1000 = 40.  (The reason you multiply by
 * 1000 rather then 100 is because some accelerometers have scale factors of
 * 12.5% so .125 * 100 = 12.5 which still is a float so .125 * 1000 = 125)
 *
 * <P><B><U> HOW THE CLASS DETERMINES TOTAL PERIOD OF DUTY CYCLE </B></U><BR>
 * When the constructor is called a method called setT2Time() is called which
 * measures the time of the low and high pulse widths and then adds them
 * together to get the total period.  The pin that it does this on is on the
 * pin which is specified for the x-axis input for the constructor.
 * So make sure that this pin is hooked up properly.  Also note, environmental
 * conditions can affect the total time so be sure to call the setT2Time()
 * function to update it regulary if this is important to you.
 * If the user wants to specify a different pin for setting the T2Time see the
 * JavaDocs on setT2Pin( pin )
 *
 *
 *
 * <P>
 * <B><U>Definitions:</B></U><BR>
 * Ty - refers to the Y axis
 * Tx - refers to the X axis
 * T2 - length of the total cycle
 * <P> When the constructor is called the T2 (total time) of the duty cycle is
 * set automatically.  It's up to the user to call the setT2Time() function
 * to make sure the T2 time period is accurate.  Environmental changes can have
 * an affect on the period.
 *
 * @version <B><U>History:</B></U><BR>
 *          1.00 - Initial Release
 *          1.01 - Optimized GForce equation
 *          1.02 - Changed class from ADXL202EB to Accelerometer
 *          1.03 - Adjust GForce equation and added scaleFactor data member
 *                 Added alternate constructors to support scaleFactor
 *          1.04 - fixed a bug related to timeout
 * @author Blake Robertson
 */
package stamp.peripheral.mems;
import stamp.core.*;


public class Accelerometer {

  // DATA Members
  private int txPin;   // Pin connected to the X axis DCM output (Header D)
  private int tyPin;   // Pin connected to the Y axis DCM output (Header C)
  private int t2Pin;   // Pin which is the total length will be determined from
  private int t2Time;  // stores the length of the total cycle
  private int timeout; // variable for customizing the timeout for pulseIn cmds

  private int scaleFactor; // variable set to match specifications of the
                           // accelerometer the class is being used for.  The
                           // scale factor can be found in the datasheet where
                           // it gives you the equation for finding the gForce.
                           // A(g) = ( dutyFactor - .5 ) / scaleFactor

  static private String gStr;  // Used to output the gForce in string format
  static private StringBuffer gStrBuf; // helper variable for gStr

 /**
  * Constructor which takes as arguments the pin numbers connected to the two
  * duty control outputs (headers D and C) of the ADXL202EB.  To measure the
  * length of the entire duty cycle the pulse width of the low cycle and the
  * pulse width of the high cycle of the TxPin are measured then summed.
  *
  * @param <B>TxPinIn</B> - int value which matches the pin number for the
  *                  X axis duty cycle output.  (This pin should be connected
  *                  to net D on the evaluation board) <BR>
  *        <B>TyPinIn</B> - int value which matches the pin number for the
  *                  Y axis duty cycle output. (This pin should be connected
  *                  to net C on the evaluation board) <BR>
  *
  */
  public Accelerometer( int txPinIn, int tyPinIn )
  {
    // init function is used to avoid code duplication for the constructors
    init( txPinIn, tyPinIn, 125 ); // sets the T2_Pin to the X_PinIn
  }

 /**
  * Alternate Constructor that allows user to specify scaleFactor
  * other then the default 12.5%
  */
  public Accelerometer( int txPinIn, int tyPinIn, int scaleFactorIn )
  {
    // init function is used to avoid code duplication for the constructors
    init( txPinIn, tyPinIn, scaleFactorIn );
  }

 /**
  * Private method which is called by the constructors to initialize common
  * data members.
  */
  private void init(  int txPinIn, int tyPinIn, int scaleFactorIn )
  {

    txPin = CPU.pins[ txPinIn ];
    tyPin = CPU.pins[ tyPinIn ];
    t2Pin = CPU.pins[ txPinIn ];  // By default t2Pin is set to the txPin
                                  // see setT2Pin() to specify a different pin

    timeout = 10000; // Sets the default timeout.  Use set method to change.

    setT2Time(); // Constructor calls this so that t2Time is automatically
                 // set.  User is responsible for calling this method to
                 // compensate for affects different environmental conditions
                 // may impose.  Recommended to be updated every 5 minutes.

    scaleFactor = 1000 / scaleFactorIn;

    gStr = "123456";      // sets string to 6 chars needed for String.valueOf
    gStrBuf = new StringBuffer(6); // initializes helper string variable
  }

  // -=-=-=-=-=-=-=-=-=-=-=-= PUBLIC METHODS =-=-=-=-=-=-=-=-=-=-=-=- //

 /**
  * Sets the total cycle time so that the duty cycle can be determined.
  *<P><B>PRECONDITIONS:</B> t2Time pin must have been set properly when
  *calling the constructor.  Typically, the t2Time pin is the Tx or Ty
  *pin.  The whole cycle is computed by totaling the pulse width's of both
  *the high and low portions.
  *<P>Duty Cycle = ( On_Time / Total_Time ) * 100
  */
  public void setT2Time( )
  {
    t2Time = (CPU.pulseIn(timeout, t2Pin, true) +
              CPU.pulseIn(timeout, t2Pin, false)) ;
  }


 /**
  * Calculates the acceleration which is determined by the interpretation of
  * the duty cycle outputs of the accelerometer for the particular
  * axis specified by the functions arguments. <P>
  *
  * This method returns a integer representation of the gForce (see returns).
  * Use with toGForceStr to get a string which contains the floating point
  * representation of the gForce suitable for output to an LCD or terminal.
  *
  * @param  axis - either 'x' or 'y' selects which axis is desired for
  *                the calculation of acceleration.
  * @return an integer representation of the gForce.
  * <P> <B>Examples:</B><BR>
  * if the gForce was 1.17 the integer representation would be 117<BR>
  * if the gForce was 0.24 the integer representation would be 24 <BR>
  * if the gForce was -.01 the integer representation would be -1 <BR>
  */
  public int findGForce( char axis )
  {
    int df = findDutyFactor(axis);  // Determines Duty factor needed for g calc

    return ( ( df - 50) * 8 );  // equation to determine gForce from
                                // the duty cycle outputs.
  }

 /**
  * This method is used to convert the integer representation of gForce to a
  * string which contains a floating point representation useful for displaying
  * to a LCD or terminal program.
  *
  * @param  int g - is the int value that is obtained by calling 'findGForce()'
  *                 see findGForce() for more information.
  *
  * @return a string which represents the current acceleration in the form
  *         #.## (where # is a digit) in the range +/- 2.00.<BR><BR>
  * <B>NOTE:</B> a reference to a string what is returned.  To save the value of
  * the string you need to call the string copy constructor with new preceeding.
  * (ie String str = new String( adxl202ebOBJ.toGForceStr( myGForceInt ) ); )
  */
  static public String toGForceStr( int g )
  {
    String.valueOf( g, gStr ); // Converts int to a string
    gStrBuf.clear();           // clear's the string buffer
    gStrBuf.append( gStr );    // appends the string of nums to the empty buffer

    // Following if statements determine where the decimal point should be
    // inserted and how many zero's needed to be tacked on.
    // EX. if an integer of -1 is returned that represents a gForce of -0.01
    // therefore you have to add two zero's after the '-' and the decimal point
    // inbetween the zeros
    if( g < 0 )
    {
      if( g > -100 )                  // if from 0 to -99 add the 0 for
        gStrBuf.insert(1, '0');       // before the decimal place

      if( g > -10 )                  // if from 0 to -9 an additional decimal
        gStrBuf.insert(1, '0');      // point must be added

      gStrBuf.insert(2, '.');        // places decimal place
    }

    else
    {
      if( g < 100 )                  // if from 0 to 99 add the 0 for before
        gStrBuf.insert(0, '0');      // the decimal place.

      if( g < 10 )                   // if from 0 to 9 an additional decimal
        gStrBuf.insert(0, '0');      // point must be added


      gStrBuf.insert(1, '.');        // inserts decimal place
    }

    return gStrBuf.toString();       // returns the string
  }


  /**
   * Calculates the duty factor of duty controlled output of the ADXL202EB for
   * the specified axis (see parameters).
   *
   *
   * @param    axis is a char which should be passed as 'x' or 'y' to select
   *           which axis the acceleration is trying to be measured for.
   * @return   an integer in the range from 0-100 which represents the
   *           percentage of time the duty cycle output pin was high compared
   *           to the total time.
   */
  public int findDutyFactor( char axis )
  {
   int onTime = 0;

    switch (axis)
    {
      case 'x':   onTime = findPulseWidth( txPin );
                  break;

      case 'y':   onTime = findPulseWidth( tyPin );
                  break;

      default:    System.out.println("Error: Invalid Axis");
                  break;
    }

    // Longest onTime interval is 32,767/50 = 655 * 8.62uS
    return ( (50 * onTime) / (t2Time / 2) );
  }

 /**
  * Accessor method
  * @return int value in the private data member txPin
  */
  public int getTxPin()
  {
    return txPin;
  }


 /**
  * Accessor method
  * @return int value stored in the private data member tyPin
  */
  public int getTyPin()
  {
    return tyPin;
  }


 /**
  * Accessor method
  * @return int value stored in the private data member t2Pin
  */
  public int getT2Pin()
  {
    return t2Pin;
  }


 /**
  * Accessor method
  * @return int value stored in the private data member t2Time
  */
  public int getT2Time()
  {
    return t2Time;
  }


 /**
  * Accessor method
  * @return int value stored in the private data member timeout
  */
  public int getTimeout()
  {
    return timeout;
  }


 /**
  * sets the txPin private data member.
  *
  * @param   The argument pinIn should be the integer representing
  *          the pin number.&nbsp;&nbsp;
  *          Ex - To set pin to pin6 you should do <I>"setTxPin(6);"</I>
  *          <B>&nbsp;&nbsp;-NOT-</B>&nbsp;&nbsp;
  *          <I> "setTxPin( CPU.pins[6] );" </I>
  */
  public void setTxPin( int pinIn )
  {
    txPin = pinIn;
  }


 /**
  * sets the tyPin private data member.
  *
  * @param   The argument pinIn should be the integer representing
  *          the pin number.&nbsp;&nbsp;
  *          Ex - To set pin to pin6 you should do <I>"setTyPin(6);"</I>
  *          <B>&nbsp;&nbsp;-NOT-</B>&nbsp;&nbsp;
  *          <I> "setTyPin( CPU.pins[6] );" </I>
  */
  public void setTyPin( int pinIn )
  {
    tyPin = pinIn;
  }


 /**
  * sets the t2Pin private data member.  Allows the user to specify which pin
  * to get a reading for the total duty cycle time.
  * This constructor was included in case a user might only be using the Y
  * axis of the ADXL202EB or to provide flexibility for another application.
  * Typically, the T2_Time pin
  * is the Tx or Ty pin.
  *
  * @param   The argument pinIn should be the integer representing
  *          the pin number.&nbsp;&nbsp;
  *          Ex - To set pin to pin6 you should do <I>"setT2Pin(6);"</I>
  *          <B>&nbsp;&nbsp;-NOT-</B>&nbsp;&nbsp;
  *          <I> "setT2Pin( CPU.pins[6] );" </I>
  */
  public void setT2Pin( int pinIn )
  {
    t2Pin = pinIn;
  }


 /**
  * sets the timeout private data member.  Normal users of the class will
  * not need to adjust this data member.  It is provided in the event that the
  * default timeout of 10000 isn't suitable for their application.
  *
  * @param   units are 8.72uS see pulseIn documentation in Javelin stamp
  *          manual to determine timeout values
  */
  public void setTimeout( int timeIn )
  {
    timeout = timeIn;
  }



   // -=-=-=-=-=-=-=-=-=-=-= PRIVATE METHODS =-=-=-=-=-=-=-=-=-=-=-=- //

 /**
  * Private method findPulseWidth returns the width of the positive cycle
  * of the pulse on the portPin specified
  */
  private int findPulseWidth( int cpuPortPin )
  {
    return CPU.pulseIn(timeout, cpuPortPin, true);
  }

  // Other private method init() is listed under the constructors


} // End of Class ADXL202E