/**
 * Class for the Memsic 2125 accelerometer
 *
 * extends the accelerometer class and adds methods to format readings
 * of the temperature output pin of the Memsic2125 when connected to
 * an Analog to Digital converter such as the LTC1298 or ADC0831.
 *
 * <P>The Memsic2125 requires no additional external components to get
 * acceleration readings from the chip.  If you want take advantage of
 * the tempature output pin of the Memsic2125 you will need to connect an
 * A/D converter to P1 of the Memsic2125.  See the A/D Abstract class to
 * learn more about how to do this.  Once you've got a reading in mV you
 * can use the methods formatTempC() and formatTempF() to determine the
 * actual celcius and fahrenheit tempature.
 * <P><U>NETLIST for Headers on Memsic2125</U>            <BR>
 * Pin 1 (Temp Out) -> Connect to A/D converter if desired<BR>
 * Pin 2 (Y-Axis Output) -> Pin 9 on Javelin Stamp        <BR>
 * Pin 3 -> GND (Vss)                                     <BR>
 * Pin 4 -> GND (Vss)                                     <BR>
 * Pin 5 (X-Axis Output) -> Pin 8 on Javelin Stamp        <BR>
 * Pin 6 -> +5V (Vcc)                                     <BR>
 *
 * @version - 1.00 Initial test file
 * @author  - Blake Robertson
 **/

package stamp.peripheral.mems;

import stamp.core.*;
import stamp.peripheral.mems.Accelerometer;

public class Memsic2125 extends Accelerometer
{
  private int tempPin;

 /**
  * Default Constructor
  */
  public Memsic2125( int txPinIn, int tyPinIn )
  {
    super( txPinIn, tyPinIn, 125 );  // calls base class constructor
                                     // third argument 125 is the integer
                                     // representation of the scale factor of
                                     // 12.5% of the Memsic2125 accelerometer.
    tempPin = -1;
  }


 /**
  * Constructor allows for temperature readings to be read in
  * NOTE: Maybe removed once tempature reading test files are written.
  */
  public Memsic2125( int txPinIn, int tyPinIn, int tempPinIn )
  {
    super( txPinIn, tyPinIn, 125 );  // calls base class constructor
                                     // third argument 125 is the integer
                                     // representation of the scale factor of
                                     // 12.5% of the Memsic2125 accelerometer.
    tempPin = tempPinIn;
  }

  static public int formatTempC( int mVolts )
  {
    int postCalib = mVolts - 1250;
    int addDegrees = postCalib / 5;

    if( (postCalib % 5) > 2 )  // Rounds up
      addDegrees++;

    return 25 + addDegrees;
  }

  static public int formatTempF( int mVolts )
  {
    int degCel = formatTempC( mVolts );

    return ((degCel * 9) / 5) + 32;
  }
}